==> file4life-1.1.3-source.tgz: file4life-1.1.3/src/F4L/Actions/Info.hs <==
{- This file is part of file4life, a file backup program written in Haskell.
 - file4life is Copyright (C) Antonio Bellezza 2014
 - file4life is free software: you can redistribute it and/or modify
 - it under the terms of the GNU General Public License as published by
 - the Free Software Foundation, either version 3 of the License, or
 - (at your option) any later version.

 - file4life is distributed in the hope that it will be useful,
 - but WITHOUT ANY WARRANTY; without even the implied warranty of
 - MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 - GNU General Public License for more details.

 - You should have received a copy of the GNU General Public License
 - along with file4life.  If not, see <http://www.gnu.org/licenses/>.
 -}
module F4L.Actions.Info (
    coverage
  , dumpFacts
  , dumpOps
  , listStorageContents
  , stats
  ) where

import F4L
import F4L.Actions.Common
import F4L.Actions.Ops
import F4L.BufferMonad
import F4L.SeenTree as ST hiding ( removeFile )
import F4L.F4LMonad
import F4L.FSMonad hiding ( outputLineMsg, cleanLine )
import F4L.Knowledge
import F4L.Utils hiding( fileSize )

import Control.Lens
import qualified Data.ByteString.Lazy as BL
import Data.List( foldl', sort, isPrefixOf )
import qualified Data.Map.Strict as M -- TODO: just use SM
import qualified Data.Map.Strict as SM
import qualified Data.Set as S



dumpOps :: ( FSMonad m, BufferMonad m, Knowledge k ) => BL.ByteString -> TopF4LMonad k m ()
dumpOps = mapM_ ( outputMsg . show ) . map snd . readVOps


dumpFacts :: ( FSMonad m, BufferMonad m, Knowledge k ) => BL.ByteString -> TopF4LMonad k m ()
dumpFacts = loadOpsDo ( const $ return () ) ( outputMsg . show )


stats :: ( FSMonad m, BufferMonad m, Knowledge k, FactCounterClass k, FileSetClass k ) 
         => GenF4LMonad k s m [(String, String)]
stats = do
  count <- use $ st_knowledge . fact_counter
  files <- use $ st_knowledge . fs_files
  return
    [ ( "facts", hShow count )
    , ( "files", hShow $ S.size files )
    , ( "total size", hShow $ totalSize files )
    ]
    where
      totalSize = sum . map fid_length . S.toList



coverage :: ( FSMonad m, KnownDataClass k ) => GenF4LMonad k s m [(Int, Int, Float)]
coverage = do
  dat <- use $ st_knowledge . kd_files . to SM.elems
  let byCopies = foldl' ( \ a d -> SM.insertWith (+) (S.size $ d ^. fd_seen ) 1 a ) SM.empty dat
  let total = fromIntegral $ length dat
  return $ sort [(i, c, 100.0 * fromIntegral c / total) | (i, c) <- SM.toList byCopies]

-- | Get storage contents by initial part of storage id
listStorageContents :: ( FSMonad m, BufferMonad m ) => String -> TopF4LMonad KnownData m [([FilePath], FileId)]
listStorageContents psid = do
  onErrorDie $ fromDB loadOps
  fullData <- use st_knowledge
  let contents = view kd_contents fullData
  case M.toList $ M.filterWithKey (const . ( psid `isPrefixOf` )) contents of
    [ ( _sid, dat ) ] ->
      return $ allFiles dat
    [] ->
      error "No storage with this ID found"
    xs ->
      error $ "Multiple storages found: " ++ show ( map fst xs )

{-
debugState :: ( FSMonad m, MonadIO m, Show k ) => F4LState k -> F4LMonad k m ()
debugState st = do
  say Quiet $ " " ++ ( show $ length $ show st )
  liftIO $ threadDelay 100000
  return ()
-}

==> file4life-1.1.4-source.tgz: file4life-1.1.4/src/F4L/Actions/Info.hs <==
{- This file is part of file4life, a file backup program written in Haskell.
 - file4life is Copyright (C) Antonio Bellezza 2014
 - file4life is free software: you can redistribute it and/or modify
 - it under the terms of the GNU General Public License as published by
 - the Free Software Foundation, either version 3 of the License, or
 - (at your option) any later version.

 - file4life is distributed in the hope that it will be useful,
 - but WITHOUT ANY WARRANTY; without even the implied warranty of
 - MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 - GNU General Public License for more details.

 - You should have received a copy of the GNU General Public License
 - along with file4life.  If not, see <http://www.gnu.org/licenses/>.
 -}
module F4L.Actions.Info (
    coverage
  , dumpFacts
  , dumpOps
  , listStorageContents
  , stats
  ) where

import F4L
import F4L.Actions.Common
import F4L.Actions.Ops
import F4L.BufferMonad
import F4L.SeenTree as ST hiding ( removeFile )
import F4L.F4LMonad
import F4L.FSMonad hiding ( outputLineMsg, cleanLine )
import F4L.Knowledge
import F4L.Utils hiding( fileSize )

import Control.Lens
import qualified Data.ByteString.Lazy as BL
import Data.List( foldl', sort, isPrefixOf )
import qualified Data.Map.Strict as M -- TODO: just use SM
import qualified Data.Map.Strict as SM
import qualified Data.Set as S



dumpOps :: ( FSMonad m, BufferMonad m, Knowledge k ) => BL.ByteString -> TopF4LMonad k m ()
dumpOps = mapM_ ( mapM_ $ outputMsg . show ) . map snd . readVOps


dumpFacts :: ( FSMonad m, BufferMonad m, Knowledge k ) => BL.ByteString -> TopF4LMonad k m ()
dumpFacts = loadOpsDo ( const $ return () ) ( outputMsg . show )


stats :: ( FSMonad m, BufferMonad m, Knowledge k, FactCounterClass k, FileSetClass k ) 
         => GenF4LMonad k s m [(String, String)]
stats = do
  count <- use $ st_knowledge . fact_counter
  files <- use $ st_knowledge . fs_files
  return
    [ ( "facts", hShow count )
    , ( "files", hShow $ S.size files )
    , ( "total size", hShow $ totalSize files )
    ]
    where
      totalSize = sum . map fid_length . S.toList


coverage :: ( FSMonad m, KnownDataClass k ) => GenF4LMonad k s m [(Int, Int, Float)]
coverage = do
  dat <- use $ st_knowledge . kd_files . to SM.elems
  let byCopies = foldl' ( \ a d -> SM.insertWith (+) (S.size $ d ^. fd_seen ) 1 a ) SM.empty dat
  let total = fromIntegral $ length dat
  return $ sort [(i, c, 100.0 * fromIntegral c / total) | (i, c) <- SM.toList byCopies]


-- | Get storage contents by initial part of storage id
listStorageContents :: ( FSMonad m, BufferMonad m ) => String -> TopF4LMonad KnownData m [([FilePath], FileId)]
listStorageContents psid = do
  onErrorDie $ fromDB loadOps
  fullData <- use st_knowledge
  let contents = view kd_contents fullData
  case M.toList $ M.filterWithKey (const . ( psid `isPrefixOf` )) contents of
    [ ( _sid, dat ) ] ->
      return $ allFiles dat
    [] ->
      error "No storage with this ID found"
    xs ->
      error $ "Multiple storages found: " ++ show ( map fst xs )


{-
debugState :: ( FSMonad m, MonadIO m, Show k ) => F4LState k -> F4LMonad k m ()
debugState st = do
  say Quiet $ " " ++ ( show $ length $ show st )
  liftIO $ threadDelay 100000
  return ()
-}
